// AUTO-GENERATED BY UpdateGenVersion.bat b88f3329130187f7944e0b6c8750aaa0c2a8fd6a
const char GenVersion_Version[] = "0.4.4-dev";
const char GenVersion_Commit[] = "b88f3329130187f7944e0b6c8750aaa0c2a8fd6a";
